-- main.lua
-- ****** Comment *****************************************************
-- Issued date : Sept.26,2024
-- Auther  : Tanzai
-- Contents: This script controlls a plugin, "H3-22S_launch".
--           This Plugin shall be stored in /plugins/ folder under airplane
--           folder.
-- Notice 1: The package may be redistributed for non commercial purposes
--           only; derivatives MAY NOT be redistributed.
-- Notice 2: Do not resize X-Plane window during flight to prevent incorrrect
--           display. When rocket is in flying, it forcibly returns to
--           preflight position. Or when rocket is on ground, window layout
--           deforms.
--           At any rate you can restore the window by pressing "Reload the
--           Current Airplane (Skip Art Reload)" under "Developer" folder.
-- *******************************************************************

setAircraftPanelRendering(false)
set3DRendering(true)
setInteractivity(true)

-- Subwindow setting  ------------------------------------------
winW = globalPropertyi("sim/graphics/view/window_width")
winH = globalPropertyi("sim/graphics/view/window_height")
local starter = contextWindow {
--     name			= "TextArea",
    position		= { 0, get(winH)-80, 1200, 80},
    noResize		= false,
    visible			= true,
	proportional	= false,
    noBackground    = true,
	layer			= SASL_CW_LAYER_FLOATING_WINDOWS,
    noDecore        = true,
    noMove          = true,
    components		= {simBoard  {position={0, 0,1200, 75},},
 					   simLiftoff{position={0, 0,1200, 34},},
					  },}

local footer = contextWindow {
--     name			= "Footer",
    position		= { 0, 0, 200, 80},
    noResize		= true,
    visible			= true,
	proportional	= false,
    noBackground    = true,
	layer			= SASL_CW_LAYER_FLOATING_WINDOWS,
    noDecore        = true,
    noMove          = true,
    components		= {simFooter{position={0, 0,200, 80},},
					  },}

-- Variable setting  ---------------------------------------------------
defineProperty("startIsOn", 0)
defineProperty("simStage", 0)
defineProperty("resetIsOn", 0)
defineProperty("t3", 0)
defineProperty("targetH", 400)
defineProperty("headPsi", 90)                                  -- deg 方向
defineProperty("pitchThe", 0)                                  -- deg 迎角
defineProperty("rollPhi", 0)                                   -- deg 左右傾
defineProperty("Note1", " ")                                   -- 説明書き、１行目
defineProperty("Note2", " ")                                   -- 説明書き、２行目
defineProperty("crrVelo", 0.0)                                 -- 1秒毎の速度
defineProperty("endT3", 0)
defineProperty("endVelo", 0.0)
defineProperty("endPosH", 0.0)
defineProperty("endLon", 0.0)
defineProperty("endLat", 0.0)

addSearchResourcesPath ( moduleDirectory .. "/configuration/" )
Booster_1_Obj = sasl.loadObject( "Booster_1.obj" )
Booster_2_Obj = sasl.loadObject( "Booster_2.obj" )
First_Stage_Obj = sasl.loadObject( "First_Stage.obj" )
Second_cover_Obj = sasl.loadObject( "Second_cover.obj" )
Fairing_SL_Obj = sasl.loadObject( "Fairing_SL.obj" )
Fairing_SR_Obj = sasl.loadObject( "Fairing_SR.obj" )
Second_Stage_Obj = sasl.loadObject( "Second_Stage.obj" )
Orbiter_Obj = sasl.loadObject( "Orbiter.obj" )
LauncherObj = sasl.loadObject( "H3_Launch_pad.obj" )
H3_sound = sasl.al.loadSample ( "H3_sound.wav" )
as = globalPropertyd("sim/flightmodel/position/true_airspeed")   -- n, m/s 速度
elv = globalPropertyd("sim/flightmodel/position/elevation")      -- n, deg 高度
lat = globalPropertyf("sim/flightmodel/position/latitude")       -- n, deg 緯度
lon = globalPropertyf("sim/flightmodel/position/longitude")      -- n, deg 経度
local quat = globalPropertyfa("sim/flightmodel/position/q")      -- y, 四元数
local vx = globalPropertyf("sim/flightmodel/position/local_vx")  -- y, m/s X軸
local vy = globalPropertyf("sim/flightmodel/position/local_vy")  -- y, m/s Y軸
local vz = globalPropertyf("sim/flightmodel/position/local_vz")  -- y, m/s Z軸
local planeX = globalPropertyf("sim/flightmodel/position/local_x")
local planeY = globalPropertyf("sim/flightmodel/position/local_y")
local planeZ = globalPropertyf("sim/flightmodel/position/local_z")
local showPart = globalPropertyia("sim/cockpit2/switches/custom_slider_on")
-- reloadAcf = sasl.findCommand("sim/operation/reload_aircraft_no_art")
-- reloadScn = sasl.findCommand("sim/operation/reload_scenery")
simTime = sasl.createTimer()
oldVelo = {0.0,0.0,0.0}
xFactor = {}
yFactor = {}
vFactor = {}
fuelMass = {}
iniMass = {}
totalMass = {}
firingT = {}
dryMass = {}
fuelRate = {}
gLsp = {}
tStart = {}
angThe = {}

-- Function to set rocket parameters  -----------------------------
function setRocket()
	if get(targetH) == 100 then                    -- 目標高度(km)
		-- Spec    Booster   1st     2nd  Orbiter
		tStart  ={    0  ,  110  ,   138 ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={  105  ,   20  ,   370 ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={   80.0,   10.0,    3.0,  0.0 }  -- 迎角 deg
		targetV = 7844                             -- 目標速度(m/sec)
	elseif get(targetH) == 200 then                -- 目標高度(km)
		-- Spec    Booster  1st      2nd  Orbiter
		tStart  ={    0  ,  110  ,   136 ,  0   }  -- 燃焼開始秒 sec
		firingT ={  105  ,   20  ,   370 ,  0   }  -- 燃焼時間 sec
		angThe  ={   80.0,   70.0,    40 ,  0.0 }  -- 迎角 deg
		targetV = 7784                             -- 目標速度(m/sec)
	elseif get(targetH) == 400 then                -- 目標高度(km)
		-- Spec    Booster   1st     2nd  Orbiter
		tStart  ={    0  ,  110  ,  206  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={  105  ,   90  ,  370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={   80.0,   50.0,   30.0,  0.0 }  -- 迎角 deg
		targetV = 7668                             -- 目標速度(m/sec)
	elseif get(targetH) == 600 then                -- 目標高度(km)
		-- Spec    Booster   31st     2nd  Orbiter
		tStart  ={    0  ,  110  ,  226  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={  105  ,   90  ,  370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={   80.0,   65.0,   30.0,  0.0 }  -- 迎角 deg
		targetV = 7558                             -- 目標速度(m/sec)
	elseif get(targetH) == 800 then                -- 目標高度(km)
		-- Spec    Booster   1st     2nd  Orbiter
		tStart  ={    0  ,  110  ,  256  ,  0   }  -- 燃焼開始時間(整数) sec
		firingT ={  105  ,   95  ,  370  ,  0   }  -- 燃焼時間(整数) sec
		angThe  ={   80.0,   70.0,   60.0,  0.0 }  -- 迎角 deg
		targetV = 7452                             -- 目標速度(m/sec)
	end
	-- 最初の6秒間は迎角90°となります。
	-- Boost終了からFirst点火まで4秒以上開けること。
	-- First点火100秒後にFaringを分離する。
	-- First終了からSecond点火まで８秒以上開けること。
	-- Second燃焼中に高度がtargetHを超えたらに迎角を0°に変更する。
	-- Second燃焼中に速度がtargetVを超えたら停止する。
	-- 燃焼時間(firingT)の上限は、firingT={ 105, 153, 370, 0 }

	-- Spec    Booster   1st     2nd  Orbiter
	gLsp    ={ 2279.3, 4165.0, 4390.4,  0.0 }  -- 有効排気速度(=g*Lsp) m/s
	dryMass ={   37.4,   12.5,    3.0,  3.0 }  -- 空筐体重 ton
	fuelRate={  1.082, 0.7092, 0.03108, 0.0 }  -- 燃料消費速度 ton/sec

	for m = 1,4 do
		fuelMass[m] = fuelRate[m] * firingT[m]
		iniMass[m] = dryMass[m] + fuelMass[m]
	end
	totalMass[1] = iniMass[1] + iniMass[2] + iniMass[3] + iniMass[4]
	totalMass[2] = iniMass[2] + iniMass[3] + iniMass[4]
	totalMass[3] = iniMass[3] + iniMass[4]
end

-- -------  Other functions  ----------------------------------------------
function setOrient()
	psRad = math.pi / 360 * get(headPsi)                          -- rad 方向
	thRad = math.pi / 360 * get(pitchThe)                         -- rad 迎角
	phRad = math.pi / 360 * get(rollPhi)                          -- rad 左右傾
	q0 = math.cos(psRad)*math.cos(thRad)*math.cos(phRad) + math.sin(psRad)*math.sin(thRad)*math.sin(phRad)
	q1 = math.cos(psRad)*math.cos(thRad)*math.sin(phRad) - math.sin(psRad)*math.sin(thRad)*math.cos(phRad)
	q2 = math.cos(psRad)*math.sin(thRad)*math.cos(phRad) + math.sin(psRad)*math.cos(thRad)*math.sin(phRad)
	q3 = -math.cos(psRad)*math.sin(thRad)*math.sin(phRad) + math.sin(psRad)*math.cos(thRad)*math.cos(phRad)
	set(quat, q0, 1)
	set(quat, q1, 2)
	set(quat, q2, 3)
	set(quat, q3, 4)
end

function setSpeed(n)                 -- Setting speed by Tsiolkovsky's equation
	massRatio= 1.0 - fuelMass[n]*(get(t3)-tStart[n])/firingT[n]/totalMass[n]
	deltaVelo= gLsp[n] * math.log( 1 / massRatio )
	vInX = (oldVelo[n] + deltaVelo) * xFactor[n]         -- 南北方向の係数
	vInY = (oldVelo[n] + deltaVelo) * yFactor[n]         -- 東西方向の係数
	vInH = (oldVelo[n] + deltaVelo) * vFactor[n] - 9.8   -- 垂直方向係数、重力を加味する
	set(vx, vInX)
	set(vz, vInY)
	set(vy, vInH)
	vDelta = (vInX^2 + vInY^2 + vInH^2)^0.5
	set(crrVelo, vDelta)                                 -- crrVelo: 1秒毎の速度
end

function setFreeFall(n)                              -- Setting free fall speed
	vInH = vInH - 9.8                                -- 重力を加味
	vDelta = (vInX^2 + vInY^2 + vInH^2)^0.5
	set(crrVelo, ( oldVelo[n] + vDelta ))            -- crrVelo: 1秒毎の速度
end

function iniValSet()                     -- Function to set initial value
	set(t3, -5)
	set(pitchThe, 0)                            -- 迎角90°(z軸が水平)の状態
	set(rollPhi, 0)
	set(Note1, "")
	set(Note2, "")
	set(showPart, 0, 10)                        -- No show Boosters smoke/jet
	set(showPart, 0, 11)                        -- No show Stage 1 jet
	set(showPart, 0, 12)                        -- No show Stage 2 jet
	set(vx, 0.0)
-- 	set(vy, 0.0)
	set(vz, 0.0)
	vInX = 0.0
	vInY = 0.0
	vInH = 0.0
end

-- Main control ----------------------------------------------------
set(simStage, 0)
set(startIsOn, 0)
set(resetIsOn, 0)
iniValSet()                                               -- Initialize values
setRocket()                                               -- Set rocket parameters
iniPlaneX = get(planeX)
iniPlaneY = get(planeY)
iniPlaneZ = get(planeZ)
iniLon = get(lon)
iniLat = get(lat)
iniElv = get(elv)
iniPsi = get(headPsi)
config = 0                                   -- All parts shown
w0 = get(winW)
h0 = get(winH)

function update()
	if get(simStage) == 0 then
		set(vx, 0.0)
		set(vy, 0.0)
		set(vz, 0.0)

		if get(startIsOn) == 1 then                      -- "Liftoff" pressed 
			iniLon = get(lon)
			iniLat = get(lat)
			iniElv = get(elv)
			iniPsi = get(headPsi)
			sasl.resetTimer( simTime )
			sasl.startTimer( simTime )
			set(simStage, 1)
			set(startIsOn, 0)
			config = 0                                  -- All parts shown
			iniValSet()                                 -- Initialize values
			setRocket()                                 -- Set rocket parameters
			for j = 1, 3 do
				xFactor[j] =  math.cos(math.rad(angThe[j])) * math.sin(math.rad(get(headPsi)))
				yFactor[j] = -math.cos(math.rad(angThe[j])) * math.cos(math.rad(get(headPsi)))
				vFactor[j] =  math.sin(math.rad(angThe[j]))
			end
			
		elseif get(resetIsOn) == 1 then                  -- "Reset" pressed 
			sasl.commandOnce( viewSet )
			sasl.stopTimer( simTime )
			set(startIsOn, 0)
			set(resetIsOn, 0)
			config = 0                                   -- All parts shown
			set(headPsi, 90)
			sasl.placeUserAtLocation(iniLat,iniLon,iniElv,iniPsi,0.0)
			iniValSet()                                  -- Initialize values
			setRocket()                                  -- Set rocket parameters
		end

	elseif get(simStage) == 1 then                        -- Simulation starts
		t0 = sasl.getElapsedSeconds( simTime )
		t4 = get(t3) + 1
		set(t3, math.floor(t0 - 5))
		if get(t3) == t4 then                             -- 秒毎に以下を実施
			t4 = get(t3)
			if get(t3) < tStart[1] then
				set(vx, 0.0)
				set(vy, 0.0)
				set(vz, 0.0)
				oldVelo[1] = 0.0
				set(Note1, "Countdown starts")
				set(Note2, "")
			elseif get(t3) == tStart[1] then              -- Booster fires
				tEnd1 = tStart[1] + firingT[1]
				set(vx, 0.0)
				set(vz, 0.0)
				sasl.al.playSample( H3_sound )
				set(showPart, 1, 10)                      -- Boosters smoke/jet on
				set(Note1, "Lifting up by SRB-3")
				set(Note2, "untill T= 105 sec")
			elseif get(t3) < tEnd1 then                   -- Booster firing
				if get(t3) > 6 then set(pitchThe, (angThe[1]-90)) end
				setSpeed(1)
				if get(t3) <= 6 then
					set(vx, 0.0)
					set(vz, 0.0)
				end
			elseif get(t3) == tEnd1 then
				setSpeed(1)
				set(showPart, 0, 10)                      -- Boosters smoke/jet off
			elseif get(t3) < tStart[2] then               -- Booster stops
				if get(t3) == tEnd1 + 2 then config = 1 end -- Bosters removed
				setFreeFall(1)
				set(Note1, "SRB-3 released")
				set(Note2, "")
 			elseif get(t3) == tStart[2] then              -- 1st Stage fires
				tEnd2 = tStart[2] + firingT[2]
				oldVelo[2] = get(crrVelo)
				set(showPart, 1, 11)                      -- Stage 1 jet on
				set(Note1, "1ST Stage engine running")
				set(Note2, "")
			elseif get(t3) < tEnd2 then                   -- 1st Stage firing
				dThe = (angThe[1]-angThe[2]) / 5
				if get(t3) == tStart[2] + 1 then
					set(pitchThe, get(pitchThe) - dThe )
				end
				if get(t3) == tStart[2] + 2 then
					set(pitchThe, get(pitchThe) - dThe )
				end
				if get(t3) == tStart[2] + 3 then
					set(pitchThe, get(pitchThe) - dThe )
				end
				if get(t3) == tStart[2] + 4 then
					set(pitchThe, get(pitchThe) - dThe )
				end
				if get(t3) == tStart[2] + 5 then
					set(pitchThe, get(pitchThe) - dThe )
				end
				if get(t3) == tStart[2] + 100 then
					config = 2
					set(Note1, "Payload fairing released")
					set(Note2, "")
				end
				setSpeed(2)
			elseif get(t3) == tEnd2 then                  -- 1st Stage firing
				setSpeed(2)
				set(showPart, 0, 11)                      -- Stage 1 jet off
			elseif get(t3) < tStart[3] then               -- 1st Stage stops
				setFreeFall(2)
				if get(t3) == tEnd2 + 2 then config = 3 end -- 1st Stage removed
				if get(t3) == tEnd2 + 4 then config = 4 end -- 2nd cover removed
				set(Note1, "1st Stage engine cutoff")
				set(Note2, "Releasing 1st Stage")
			elseif get(t3) == tStart[3] then              -- 2nd Stage fires
				tEnd3 = tStart[3] + firingT[3]
				oldVelo[3] = get(crrVelo)
				set(pitchThe, (angThe[3]-90))
				set(showPart, 1, 12)                      -- Stage 2 jet on
				set(Note1, "2nd Stage engine running")
				set(Note2, "")
			elseif get(t3) < tEnd3 then                   -- 2nd Stage running
				setSpeed(3)
				set(endPosH, math.floor(get(elv)/1000))
				if get(endPosH) > get(targetH) then   -- Pitch turns to 0 deg
					set(pitchThe, -90)
					set(vy, 0.0)
					set(Note1, "Got around to target height")
					set(Note2, "Increasing speed")
				end
				if get(crrVelo) > targetV then            -- Pitch turns to 0 deg
					set(showPart, 0, 12)                  -- Stage 2 jet off
					set(simStage, 2)                      -- Simulation ends
					set(endT3, get(t3))
					set(endVelo, math.floor(get(crrVelo)))
					set(endPosH, math.floor(get(elv)/100)/10)
					set(endLon, get(lon))
					set(endLat, get(lat))
				end
			elseif get(t3) == tEnd3 then                  -- 2nd Stage stops
				setSpeed(3)
				set(showPart, 0, 12)                      -- Stage 2 jet off
			elseif get(t3) == tEnd3 + 2 then              -- 2nd Stage removed
				config = 5                                -- 2nd Stage removed
				set(simStage, 2)                          -- Simulation ends
				set(endT3, get(t3))
				set(endVelo, math.floor(get(crrVelo)))
				set(endPosH, math.floor(get(elv)/100)/10)
				set(endLon, get(lon))
				set(endLat, get(lat))
			end
			updateAll( components )
		end
		setOrient()		
	end
	
	if w0 ~= get(winW) or h0 ~= get(winH) then
		starter:setPosition(0, h0 - 80, 1200, 80)
		footer :setPosition(0, 0, 200, 80)

		if get(simStage) ~= 0 then
			set(simStage, 0)
			set(startIsOn, 0)
			set(resetIsOn, 0)
			config = 0                                   -- All parts shown
			set(headPsi, 90)
			sasl.placeUserAtLocation(iniLat,iniLon,iniElv,iniPsi,0.0)
			iniValSet()                                  -- Initialize values
			setRocket()                                  -- Set rocket parameters
		end	

		updateAll( components )
	end
end

-- Draw rocket  ----------------------------------------------------------
draw_count = 0
function drawObjects()
	if draw_count == 0 then
        draw_count = 1
		iniPlaneX = get(planeX)
		iniPlaneY = get(planeY)
		iniPlaneZ = get(planeZ)
        Launcher = sasl.createInstance(LauncherObj)
        sasl.setInstancePosition(Launcher,iniPlaneX,iniPlaneY,iniPlaneZ,0,get(headPsi),0)
	end
	
	x, y, z = sasl.modelToLocal(0, 0, 0)
	a = get(pitchThe)
	b = get(headPsi)
	c = get(rollPhi)
	sasl.drawObject(Orbiter_Obj, x, y, z, a, b, c, 0, 0)
	
	if config == 0 then                                 -- Show All
		sasl.drawObject(Booster_1_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Booster_2_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(First_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_cover_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Fairing_SL_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Fairing_SR_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_Stage_Obj, x, y, z, a, b, c, 0, 0)
	elseif config == 1 then                             -- Boosters removed
		sasl.drawObject(First_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_cover_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Fairing_SL_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Fairing_SR_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_Stage_Obj, x, y, z, a, b, c, 0, 0)
	elseif config == 2 then                             -- Fairings removed
		sasl.drawObject(First_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_cover_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Orbiter_Obj, x, y, z, a, b, c, 0, 0)
	elseif config == 3 then                             -- 1st Stage removed
		sasl.drawObject(Second_cover_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Second_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Orbiter_Obj, x, y, z, a, b, c, 0, 0)
	elseif config == 4 then                             -- Second Cover removed
		sasl.drawObject(Second_Stage_Obj, x, y, z, a, b, c, 0, 0)
		sasl.drawObject(Orbiter_Obj, x, y, z, a, b, c, 0, 0)
	elseif config == 5 then                             -- 2bd Stage removed
		sasl.drawObject(Orbiter_Obj, x, y, z, a, b, c, 0, 0)	
	end
end

-- Set Circle view first  ------------------------------------------------
viewSet = sasl.findCommand ( "sim/view/circle" )
function onModuleInit ()
	sasl.commandOnce( viewSet )
end

function onAirportLoaded()
	draw_count = 0
	config = 0
	sasl.commandOnce( viewSet )
end

components={ simBoard{}, simLiftoff{}, simFooter{},}
